@@ -30,13 +30,14 @@ try:
 
     from cachemaker import loadcache, savecache, monkeycache, splitverbsubject
     from config import __version__, __description__, __projecturl__
-    from config import read_config, write_config, config_file
+    from config import read_config, write_config, config_file, default_profile
     from optparse import OptionParser
     from prettytable import PrettyTable
     from printer import monkeyprint
     from requester import monkeyrequest
     from requester import login
     from requester import logout
+    from urlparse import urlparse
 except ImportError, e:
     print("Import error in %s : %s" % (__name__, e))
     import sys
@@ -68,18 +69,19 @@ class CloudMonkeyShell(cmd.Cmd, object):
     ruler = "="
     config_options = []
     verbs = []
+    prompt = "🐵 > "
+    protocol = "http"
+    host = "localhost"
+    port = "8080"
+    path = "/client/api"
 
     def __init__(self, pname, cfile):
         self.program_name = pname
         self.config_file = cfile
         self.config_options = read_config(self.get_attr, self.set_attr,
                                           self.config_file)
-        self.credentials = {'apikey': self.apikey, 'secretkey': self.secretkey,
-                            'username': self.username,
-                            'password': self.password}
         self.loadcache()
-        self.prompt = self.prompt.strip() + " "  # Cosmetic fix for prompt
-
+        self.init_credential_store()
         logging.basicConfig(filename=self.log_file,
                             level=logging.DEBUG, format=log_fmt)
         logger.debug("Loaded config fields:\n%s" % map(lambda x: "%s=%s" %
@@ -94,6 +96,16 @@ class CloudMonkeyShell(cmd.Cmd, object):
             logger.debug("Error: Unable to read history. " + str(e))
         atexit.register(readline.write_history_file, self.history_file)
 
+    def init_credential_store(self):
+        self.credentials = {'apikey': self.apikey, 'secretkey': self.secretkey,
+                            'username': self.username,
+                            'password': self.password}
+        parsed_url = urlparse(self.url)
+        self.protocol = "http" if not parsed_url.scheme else parsed_url.scheme
+        self.host = parsed_url.netloc
+        self.port = "8080" if not parsed_url.port else parsed_url.port
+        self.path = parsed_url.path
+
     def get_attr(self, field):
         return getattr(self, field)
 
@@ -105,6 +117,7 @@ class CloudMonkeyShell(cmd.Cmd, object):
 
     def cmdloop(self, intro=None):
         print(self.intro)
+        print "Using management server profile:", self.profile, "\n"
         while True:
             try:
                 super(CloudMonkeyShell, self).cmdloop(intro="")
@@ -123,6 +136,8 @@ class CloudMonkeyShell(cmd.Cmd, object):
         for verb in self.verbs:
             def add_grammar(verb):
                 def grammar_closure(self, args):
+                    if args is None:
+                        return
                     if self.pipe_runner("%s %s" % (verb, args)):
                         return
                     if ' --help' in args or ' -h' in args:
@@ -255,10 +270,8 @@ class CloudMonkeyShell(cmd.Cmd, object):
     def make_request(self, command, args={}, isasync=False):
         response, error = monkeyrequest(command, args, isasync,
                                         self.asyncblock, logger,
-                                        self.host, self.port,
-                                        self.credentials,
-                                        self.timeout, self.protocol,
-                                        self.path, self.expires)
+                                        self.url, self.credentials,
+                                        self.timeout, self.expires)
         if error is not None:
             self.monkeyprint(error)
         return response
@@ -398,19 +411,30 @@ class CloudMonkeyShell(cmd.Cmd, object):
     def do_set(self, args):
         """
         Set config for cloudmonkey. For example, options can be:
-        host, port, apikey, secretkey, log_file, history_file
+        url, auth, log_file, history_file
         You may also edit your ~/.cloudmonkey_config instead of using set.
 
         Example:
-        set host 192.168.56.2
+        set url http://localhost:8080/client/api
         set prompt 🐵 cloudmonkey>
         set log_file /var/log/cloudmonkey.log
         """
         args = args.strip().partition(" ")
         key, value = (args[0], args[2])
-        setattr(self, key, value)  # keys and attributes should have same names
-        self.prompt = self.prompt.strip() + " "  # prompt fix
+        setattr(self, key, value)
+        if key in ['host', 'port', 'path', 'protocol']:
+            key = 'url'
+            self.url = "%s://%s:%s%s" % (self.protocol, self.host, self.port, self.path)
+            print "This option has been deprecated, please set 'url' instead"
+            print "This server url will be used:", self.url
         write_config(self.get_attr, self.config_file)
+        read_config(self.get_attr, self.set_attr, self.config_file)
+        self.init_credential_store()
+        if key.strip() == 'profile':
+            print "\nLoaded server profile '%s' with options:" % key
+            for option in default_profile.keys():
+                print "    %s = %s" % (option, self.get_attr(option))
+            print
 
     def complete_set(self, text, line, begidx, endidx):
         mline = line.partition(" ")[2]
@@ -423,17 +447,21 @@ class CloudMonkeyShell(cmd.Cmd, object):
         Login using stored credentials. Starts a session to be reused for
         subsequent api calls
         """
-        url = "%s://%s:%s%s" % (self.protocol, self.host, self.port, self.path)
-        session, sessionkey = login(url, self.username, self.password)
-        self.credentials['session'] = session
-        self.credentials['sessionkey'] = sessionkey
+        try:
+            session, sessionkey = login(self.url, self.username, self.password)
+            self.credentials['session'] = session
+            self.credentials['sessionkey'] = sessionkey
+        except Exception, e:
+            print "Error while trying to log in to the server: ", str(e)
 
     def do_logout(self, args):
         """
         Logout of session started with login with username and password
         """
-        url = "%s://%s:%s%s" % (self.protocol, self.host, self.port, self.path)
-        logout(url, self.credentials.get('session'))
+        try:
+            logout(self.url, self.credentials.get('session'))
+        except Exception, e:
+            pass
         self.credentials['session'] = None
         self.credentials['sessionkey'] = None
 
